/*
 * Copyright 2004-2006 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.remoting.axis.deployer;

import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;

import org.apache.axis.deployment.wsdd.WSDDConstants;
import org.apache.axis.deployment.wsdd.WSDDException;
import org.apache.axis.utils.XMLUtils;
import org.seasar.framework.container.ComponentDef;
import org.seasar.framework.container.MetaDef;
import org.seasar.framework.log.Logger;
import org.seasar.framework.message.MessageFormatter;
import org.seasar.framework.util.ResourceUtil;
import org.seasar.remoting.axis.DeployFailedException;
import org.seasar.remoting.axis.ServiceDef;
import org.seasar.remoting.axis.deployment.WSDDS2Service;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * dicont@CɋLqꂽR|[lgT[rXƂAxisɃfvC܂B
 * 
 * @author koichik
 */
public class ServiceDeployer implements ItemDeployer {

    // class fields
    private static final Logger logger = Logger.getLogger(ServiceDeployer.class);

    // instance fields
    protected final AxisDeployer deployer;

    /**
     * CX^X\z܂B
     * 
     * @param deployer
     *            fvC[
     */
    public ServiceDeployer(final AxisDeployer deployer) {
        this.deployer = deployer;
    }

    /**
     * R|[lgT[rXƂăfvC܂B
     * 
     * @param componentDef
     *            R|[lg`
     * @param metaDef
     *            ^`
     */
    public void deploy(final ComponentDef componentDef, final MetaDef metaDef) {
        final WSDDS2Service service = createWSDDS2Service(componentDef, metaDef);
        deployer.getDeployment(componentDef.getContainer()).deployService(service);

        if (logger.isDebugEnabled()) {
            logger.log("DAXS0003", new Object[] { service.getQName() });
        }
    }

    /**
     * <code>WSDDS2Service</code> CX^XĕԂ܂B <br>
     * ^f[^̎wɏ]A <code>ServiceDef</code> ܂WSDDt@C
     * <code>WSDDS2Service</code> CX^X܂B
     * 
     * @param componentDef
     *            R|[lg`
     * @param metaDef
     *            ^f[^`
     * @return <code>WSDDS2Service</code>
     */
    protected WSDDS2Service createWSDDS2Service(final ComponentDef componentDef,
            final MetaDef metaDef) {
        try {
            final Object metadata = metaDef.getValue();
            if (metadata == null) {
                return new WSDDS2Service(componentDef);
            }
            else if (metadata instanceof ServiceDef) {
                return new WSDDS2Service(componentDef, (ServiceDef) metadata);
            }
            else if (metadata instanceof String) {
                return new WSDDS2Service(componentDef, getServiceElement((String) metadata));
            }
            throw new DeployFailedException();
        }
        catch (final WSDDException e) {
            throw new DeployFailedException(e);
        }
    }

    /**
     * WSDDt@CNXpXǂݍ݁A <code>&lt;service&gt;</code> vfԂ܂B
     * 
     * @param wsddFileName
     *            WSDDt@C̃pX
     * @return <code>&lt;service&gt;</code> vf
     */
    protected Element getServiceElement(final String wsddFileName) {
        try {
            final InputStream is = ResourceUtil.getResourceAsStream(wsddFileName);
            final Element documentElement = XMLUtils.newDocument(is).getDocumentElement();
            final Element[] serviceElements = getChildElements(documentElement,
                    WSDDConstants.ELEM_WSDD_SERVICE);
            if (serviceElements.length != 1) {
                throw new DeployFailedException(MessageFormatter.getSimpleMessage("EAXS0005",
                        new Object[] { wsddFileName }));
            }
            return serviceElements[0];
        }
        catch (final DeployFailedException e) {
            throw e;
        }
        catch (final Exception e) {
            throw new DeployFailedException(e);
        }
    }

    /**
     * w肳ꂽ[Jqvf̔zԂ܂B
     * 
     * @param parent
     *            evf
     * @param name
     *            qvf̃[J
     * @return w肳ꂽ[Jqvf̔zBYqvf݂Ȃꍇ͋̔zB
     */
    protected Element[] getChildElements(final Element parent, final String name) {
        final List result = new ArrayList();
        final NodeList children = parent.getChildNodes();
        for (int i = 0; i < children.getLength(); i++) {
            final Node thisNode = children.item(i);
            if (thisNode instanceof Element) {
                final Element element = (Element) thisNode;
                if (element.getLocalName().equals(name)) {
                    result.add(element);
                }
            }
        }
        return (Element[]) result.toArray(new Element[result.size()]);
    }
}
